/* SCCS  @(#)nsl_diff.C	1.1---95/08/13--19:20:46 */
//
//	nsl_diff.C
//

# include "kallincl.h"

nsl_diff::nsl_diff(int fg) : nsl_public("leaky")
{
	init_diff(fg);
}
nsl_diff::~nsl_diff()
{
}

int nsl_diff::init_diff(int fg)
{
	
	set_base_type("diff");
	set_update_fg(NSL_ON);

	if (SYSTEM == NULL)
	{
		SYSTEM = new nsl_system;
		cmd_out("nsl_diff:Initializing SYSTEM...");
	}

	if (fg == 2) // add to system library
	{
		SYSTEM->add_diff_model(this);
		return 1;
	}

 	nsl_model* model = SYSTEM->get_model();

	if (model == NULL)
	{
		cmd_error("nsl_diff: Null Model");
		return 1;
	}

	nsl_diff_manager* mg = model->get_diff_manager();

	if (mg == NULL)
	{
		cmd_error("nsl_diff: Null manager");
		return 1;
	}

 	m_parent = mg;
	if (fg == 1)
 		mg->add_diff(this);
	dobj = 0;

	return 1;
}

// print 

int nsl_diff::print_diff_status(ostream& out) const
{
	nsl_layer* layer;

	print_base(out);

	if (dobj != NULL)
	{
		out << "set time_constant " << dobj->get_tm() << "\n";
		if ((layer = dobj->get_layer()) != NULL)
			layer->print_status(out);
	}

	out.flush();

	return 1;
}
int nsl_diff::print_status(ostream& out)  const // virtual
{
	print_diff_status(out);
	return 1;
}
int nsl_diff::print_diff(ostream& out) const
{
	print_base(out);
	return 1;
}
int nsl_diff::print(ostream& out) const // virtual
{
	print_diff(out);
	return 1;
}

// print all

int nsl_diff::print_diff_all_status(ostream& out) const
{
	print_diff_status(out);
	print_base_all_status(out);
	return 1;
}
int nsl_diff::print_all_status(ostream& out)  const // virtual
{
	print_diff_all_status(out);
	return 1;
}
int nsl_diff::print_diff_all(ostream& out) const
{
	print_diff(out);
	print_base_all(out);
	return 1;
}
int nsl_diff::print_all(ostream& out) const // virtual
{
	print_diff_all(out);
	return 1;
}

// read, write

int nsl_diff::write_diff(nsl_buffer&)
{
	return 1;
}
int nsl_diff::write(nsl_buffer& buf)
{
	write_diff(buf);
	return 1;
}
int nsl_diff::read_diff(nsl_buffer&)
{
	return 1;
}
int nsl_diff::read(nsl_buffer& buf)
{
	read_diff(buf);
	return 1;
}

// differential equation set

int nsl_diff::set_diff_obj(nsl_layer* layer)
{
 	// update only if new method
	if (m_parent == 0)
	{
		cmd_error("nsl_diff: Null parent");
		return 1;
	}

 	if (get_update_fg() == 0)
		return 1;

	set_update_fg(NSL_OFF);

	nsl_list* dlist;
	const char* t = layer->get_abstract_type();
	const char* m = ((nsl_diff_manager*) m_parent)->get_method();

	if ((dlist = (nsl_list*) SYSTEM->get_m_children_list()->
		get_by_name("diff_method")) == NULL)
	{
	     cmd_error("nsl_diff: NULL system diff list");
	     return 1;
	}

	nsl_string dtype;
	
	sprintf(dtype,"%s_%s",m,t);
	dobj = (nsl_diff_method*) dlist->get_by_type(dtype);

	if (dobj == NULL)
	{
		disable();
		cmd_error("nsl_diff: Bad Method Obj");
		cmd_error("nsl_diff: Layer Type: ",t);
		cmd_error("nsl_diff: Method Type: ",m);
	}

	return 1;
}
int nsl_diff::update_diff_obj(nsl_layer* m,num_type k)
{
	if (dobj != NULL)
	{
		enable();
		dobj->set_tm(k);
		dobj->set_layer(m);
		dobj->set_dt(((nsl_diff_manager*) m_parent)->get_dt());
		dobj->init();
 	}

	return 1;
}

// copy diff values locally

nsl_diff& nsl_diff::eq(nsl_layer& m,num_type k)
{
 	set_diff_obj(&m);
	update_diff_obj(&m,k);

	return *this;
}
nsl_diff& nsl_diff::eq(nsl_layer& m,nsl_data& k)
{
 	set_diff_obj(&m);
	update_diff_obj(&m,k.elem());

	return *this;
}

// call diff object according to specified method

int nsl_diff::operator=(const num_type a)
{
	if (dobj == NULL)
	{
		cmd_error("nsl_diff: Bad Method");
		return 1;
	}
	if (get_able_fg() == 1)
		dobj->exec(a);

	return 1;
}
int nsl_diff::operator=(const nsl_num_data& a)
{
	if (dobj == NULL)
	{
		cmd_error("nsl_diff: Bad Method");
		return 1;
	}
	if (get_able_fg() == 1)
		dobj->exec(a);

	return 1;
}
int nsl_diff::operator=(const nsl_num_vector& a)
{
	if (dobj == NULL)
	{
		cmd_error("nsl_diff: Bad Method");
		return 1;
	}
	if (get_able_fg() == 1)
  		dobj->exec(a);
 
	return 1;
}
int nsl_diff::operator=(const nsl_num_matrix& a)
{
	if (dobj == NULL)
	{
		cmd_error("nsl_diff: Bad Method");
		return 1;
	}
	if (get_able_fg() == 1)
		dobj->exec(a);

	return 1;
}

// added 12/95 by AW

int nsl_diff::run(nsl_layer* out,nsl_layer* in)
{
	if (dobj != NULL)
	{
		dobj->set_tm(tm.elem());
		dobj->set_dt(delta.elem());
		dobj->run(out,in);
	}
	else
		cmd_error("nsl_diff::run: Bad Method");

	return 1;
}

nsl_diff* nsl_diff::make_copy()
{
	return new nsl_diff;
}

