/* SCCS  @(#)nsl_system.C	1.1---95/10/13--16:37:16 */
//
//	nsl_system.C
//

# include "kallincl.h"

nsl_system::nsl_system() : nsl_public("system")
{
	init_system();
}
nsl_system::~nsl_system()
{
	delete model_list;
	delete proc_list;
	delete file_type_list;
	delete diff_method_list;
	delete diff_model_list;
	delete thresh_lib_list;
	delete fm;
}

int nsl_system::init_system()
{
	if (SYSTEM == NULL)
	{
		SYSTEM = this; // from nsl_root
		NSL_SYSTEM = this;
	}

	if (INTERPRETER == NULL)
	{
		INTERPRETER = new nsl_interpreter;
		NSL_INTERPRETER = INTERPRETER;
		cmd_out("nsl_system:Initializing NSL_INTERPRETER...");
	}

	char* env;

	set_base_type("system");
	set_name("system");
	m_parent = this;

	env = getenv("NSL_DIR");
	if (env != NULL)
		strcpy(nsl_dir,env);
	else
	{
		cmd_error("NSL_DIR not set\nUse `setenv NSL_DIR *dir*'");
		strcpy(nsl_dir,".");
	}

	xsystem_fg = NSL_OFF;

 	model_total = 0;
	model_list = new nsl_list("model");

	proc_total = 0;
	proc_list = new nsl_list("proc");

	file_type_total = 0;
	file_type_list = new nsl_list("file_type");

	diff_method_total = 0;
	diff_method_list = new nsl_list("diff_method");

	diff_model_total = 0;
	diff_model_list = new nsl_list("diff_model");

	thresh_lib_total = 0;
	thresh_lib_list = new nsl_list("thresh_lib");

	fm = new nsl_file_manager;

//#ifndef __GNUG__
//	log.attach(1); // set log to standard output
//#endif

	m_children_list->append(model_list);
	m_children_list->append(proc_list);
	m_children_list->append(file_type_list);
 	m_children_list->append(diff_method_list);
 	m_children_list->append(thresh_lib_list);
 	m_children_list->append(fm);

	return 1;
}

// add model
 
int nsl_system::add_model(nsl_model* model)
{
	model->set_index(++model_total);
	model_list->append(model);
	model->set_m_parent(this);

	set_ptr(model); // OBJ = model
	set_model(model);

	return model_total;
}

nsl_model* nsl_system::get_model(const char* mname)
{
	return (nsl_model*) model_list->get_by_name(mname);
}
nsl_model* nsl_system::get_model(const int mindex)
{
	return (nsl_model*) model_list->get_by_index(mindex);
}

int nsl_system::add_diff_method(nsl_diff_method* d)
{
	d->set_index(++diff_method_total);
	diff_method_list->append(d);
//	d->set_m_parent(this);

	return diff_method_total;
}

nsl_diff_method* nsl_system::get_diff_method(const char* mname)
{
	return (nsl_diff_method*) diff_method_list->get_by_name(mname);
}

int nsl_system::add_diff_model(nsl_diff* d)
{
	d->set_index(++diff_model_total);
	diff_model_list->append(d);
//	d->set_m_parent(this);

	return diff_model_total;
}

nsl_diff* nsl_system::get_diff_model(const char* mname)
{
	return (nsl_diff*) diff_model_list->get_by_name(mname);
}

int nsl_system::add_thresh_lib(nsl_thresh_library* d)
{
	d->set_index(++thresh_lib_total);
	thresh_lib_list->append(d);
//	d->set_m_parent(this);

	return thresh_lib_total;
}

nsl_thresh_library* nsl_system::get_thresh_lib(const char* mname)
{
	return (nsl_thresh_library*) thresh_lib_list->get_by_name(mname);
}

int nsl_system::init()
{
	nsl_model* model;

	model_list->wind_up();
	// initiating models (SYS_MODULE)
	while ((model = (nsl_model*) model_list->next()) != NULL)
		model->init();

	return 1;
}

// add proc
 
int nsl_system::insert_proc(nsl_module* m)
{
	proc_list->wind_up();
	proc_list->set_index(++proc_total);
	proc_list->insert(m);
	proc_list->set_m_parent(this);

 	return proc_total;
}
int nsl_system::add_proc(nsl_module* m)
{
	proc_list->set_index(++proc_total);
	proc_list->append(m);
	proc_list->set_m_parent(this);

 	return proc_total;
}
int nsl_system::init_proc()
{
 	if (proc_list != NULL)
	{
	    proc_list->wind_up();
	    nsl_module* m;
	    while (m = (nsl_module*) proc_list->next()) 
		((nsl_module*) m)->exec(); // global
	}
	else
	   cmd_out("NULL 'nsl_proc_list'");

	return 1;
}

// get model

nsl_module* nsl_system::get_proc(const char* mname)
{
	return (nsl_module*) proc_list->get_by_name(mname);
}
nsl_module* nsl_system::get_proc(const int mindex)
{
	return (nsl_module*) proc_list->get_by_index(mindex);
}

// i/o files

int nsl_system::add_file_type(nsl_file* sfile)
{
	sfile->set_index(++file_type_total);
	file_type_list->append(sfile);
	sfile->set_m_parent(this);

	return file_type_total;
}
nsl_file* nsl_system::get_file_type(const char* tname) // any dir
{
	const char* str;
	file_type_list->wind_up();
	nsl_file* sfile;
	while (sfile = (nsl_file*)file_type_list->next())
	{
		str = sfile->get_filename();	// w/o file ending
		if (strcmp(tname,str) == 0)
			return sfile;
		str = sfile->get_name(); 	// w/ file ending
		if (strcmp(tname,str) == 0)
			return sfile;
	}
	return (nsl_file*) 0;
}
nsl_file* nsl_system::get_file_type(const int tindex)
{
	return (nsl_file*) file_type_list->get_by_index(tindex);
}

// print

int nsl_system::print_system_status(ostream& out) const
{
//	print_base_status(out);
	
	out << "\n// model_total:\t\t" << model_total;
	out << "\n// diff_method_total:\t" << diff_method_total;
	out << "\n// thresh_lib_total:\t" << thresh_lib_total;
	out << "\n// file_type_total:\t" << file_type_total;
	out << "\n// nsl_dir:\t" << nsl_dir;
 	out.flush();

	return 1;
}
int nsl_system::print_status(ostream& out)  const // virtual
{
	print_system_status(out);
	return 1;
}
int nsl_system::print_system(ostream& out) const
{
//	print_base(out);

	print_system_status(out);
	
	return 1;
}
int nsl_system::print(ostream& out) const // virtual
{
	print_system(out);
	return 1;
}

// print all

int nsl_system::print_system_all_status(ostream& out) const
{
	print_system_status(out);
//	print_base_all_status(out);
	return 1;
}
int nsl_system::print_all_status(ostream& out)  const // virtual
{
	print_system_all_status(out);
	return 1;
}
int nsl_system::print_system_all(ostream& out) const
{
	print_system(out);
//	print_base_all(out);
	return 1;
}
int nsl_system::print_all(ostream& out) const // virtual
{
	print_system_all(out);
	return 1;
}

// read, write

int nsl_system::write_system(nsl_buffer&)
{
	return 1;
}
int nsl_system::write(nsl_buffer& buf)
{
	write_system(buf);
	return 1;
}
int nsl_system::read_system(nsl_buffer&)
{
	return 1;
}
int nsl_system::read(nsl_buffer& buf)
{
	read_system(buf);
	return 1;
}
