/* SCCS  @(#)nsl_matrix_prod_library.C	1.1---95/08/13--20:48:16 */
//
//	nsl_matrix_prod_library.C
//

# include "kallincl.h"

// product

nsl_data NSLprod(const nsl_vector& w,const nsl_vector& x)
{
	int size = w.get_imax();

	nsl_data c;

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
	
	num_type* wv = w.get_vector();
	num_type* xv = x.get_vector();

	for (int i = 0; i < size; i++)
		c = c + wv[i]*xv[i];

	return c;
}
nsl_vector NSLprod(const nsl_vector& w,const nsl_matrix& x)
{
	int size = x.get_imax();
	int vsize = x.get_jmax();

	nsl_vector c(vsize);

	if (size != w.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
	
	num_type** xv = x.get_matrix();
	num_type* wv = w.get_vector();
	num_type* cv = c.get_vector();

	for (int j = 0; j < vsize; j++)
	{
	    cv[j] = 0;
	    for (int i = 0; i < size; i++)
		cv[j] = cv[j] + wv[i]*xv[i][j];
	}

	return c;
}
nsl_vector NSLprod(const nsl_matrix& w,const nsl_vector& x)
{
	int size = w.get_jmax();
	int vsize = w.get_imax();

	nsl_vector c(vsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	num_type** wv = w.get_matrix();
	num_type* xv = x.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < vsize; i++)
	{
	    cv[i] = 0;
	    for (int j = 0; j < size; j++)
		cv[i] = cv[i] + wv[i][j]*xv[j];
	}

	return c;
}

nsl_matrix NSLprod(const nsl_matrix& w,const nsl_matrix& x)
{
	int isize = w.get_imax();
	int jsize = x.get_jmax();

	int size = w.get_jmax();

	nsl_matrix c(isize,jsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	num_type** wv = w.get_matrix();
	num_type** xv = x.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < isize; i++)
	    for (int j = 0; j < jsize; j++)
	    {
	    	cv[i][j] = 0;
	    	for (int k = 0; k < size; k++)
		   cv[i][j] = cv[i][j] + wv[i][k]*xv[k][j];
	    }

	return c;
}
