/* SCCS  @(#)nsl_matrix_conv_library.C	1.1---95/08/13--20:48:15 */
//
//	nsl_matrix_conv_library.C
//

# include "kallincl.h"

// matrix convolution

nsl_matrix operator*(const nsl_matrix& a,const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (matrix) " << a.get_name() << " * (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

	nsl_matrix c(sbimax,sbjmax);
	nsl_matrix d(simax,sjmax);
	num_type **Va = a.get_matrix();
	num_type **Vc = c.get_matrix();
	num_type **Vd = d.get_matrix();

	d.put_sector(b,saimax/2,sajmax/2); 

	for (int i = 0; i < sbimax; i++)
	   for (int j = 0; j < sbjmax; j++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + Va[m][n] * Vd[i+m][j+n];
		Vc[i][j] = val;
	   }

	return c;
}
nsl_matrix operator*(const nsl_vector& a,const nsl_matrix& b) // row,col vector
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (vector) " << a.get_name() << " * (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);
	num_type **Vc = c.get_matrix();
	num_type  *Va = a.get_vector();

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		num_type **Vd = d.get_matrix();
		d.put_sector(b,0,saimax/2); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i][j+m];
			Vc[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		num_type **Vd = d.get_matrix();
		d.put_sector(b,saimax/2,0); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i+m][j];
			Vc[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix operator*(const nsl_matrix& b,const nsl_vector& a) // same as previous
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (matrix) " << a.get_name() << " * (vector) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);
	num_type **Vc = c.get_matrix();
	num_type  *Va = a.get_vector();

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		num_type **Vd = d.get_matrix();

		d.put_sector(b,0,saimax/2); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i][j+m];
			Vc[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		num_type **Vd = d.get_matrix();

		d.put_sector(b,saimax/2,0); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i+m][j];
			Vc[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix operator*(const nsl_matrix& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (matrix) " << a.get_name() << " * (data) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);
	num_type **Va = a.get_matrix();
	num_type **Vc = c.get_matrix();
	num_type   Vb = b.get_data();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		Vc[i][j] = Va[i][j] * Vb;

	return c;
}
nsl_matrix operator*(const nsl_data& a, const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (data) " << a.get_name() << " * (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);
	num_type **Vb = b.get_matrix();
	num_type **Vc = c.get_matrix();
	num_type   Va = a.get_data();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		Vc[i][j] = Va * Vb[i][j];

	return c;
}
nsl_matrix operator*(const nsl_matrix& a,const num_type b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (matrix) " << a.get_name() << " * (num_type) ";
        cout << b << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);
	num_type **Va = a.get_matrix();
	num_type **Vc = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		Vc[i][j] = Va[i][j] * b;

	return c;
}
nsl_matrix operator*(const num_type a, const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator*\n";
        cout << "// (num_type) " << a << " * (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);
	num_type **Vb = b.get_matrix();
	num_type **Vc = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		Vc[i][j] = a * Vb[i][j];

	return c;
}

// zero edge matrix convolution 

nsl_matrix NSLconv_zero(const nsl_matrix& a,const nsl_matrix& b)
{
	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

	nsl_matrix c(sbimax,sbjmax);
	nsl_matrix d(simax,sjmax);

	d.put_sector(b,saimax/2,sajmax/2); 

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

	for (int i = 0; i < sbimax; i++)
	   for (int j = 0; j < sbjmax; j++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
	   }

	return c;
}
nsl_matrix NSLconv_zero(const nsl_vector& a,
	const nsl_matrix& b) // row,col vector
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(const nsl_matrix& b,
	const nsl_vector& a) // same as previous
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(const nsl_matrix& a,const nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type bv = b.get_data();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * bv;

	return c;
}
nsl_matrix NSLconv_zero(const nsl_data& a, const nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type av = a.get_data();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = av * bv[i][j];

	return c;
}
nsl_matrix NSLconv_zero(const nsl_matrix& a,const num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * b;

	return c;
}
nsl_matrix NSLconv_zero(const num_type a,const  nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = a * bv[i][j];

	return c;
}

// wrap around convolution

nsl_matrix NSLconv_wrap(const nsl_matrix& a,const nsl_matrix& b)
{
      	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
      	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int smi = saimax/2;
	int smj = sajmax/2;

 	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

      	nsl_matrix c(sbimax,sbjmax);
      	nsl_matrix d(simax,sjmax);

      	d.put_sector(b,saimax/2,sajmax/2); 

	int i,j;

	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

      	for (i = smi-1; i >= 0; i--)
    	   for (j = smj; j < sbjmax+smj; j++)
 	   	dv[i][j] = bv[(i-smi+sbimax)%sbimax][j-smj];
      	for (i = smi+sbimax; i < simax; i++)
    	   for (j = smj; j < sbjmax+smj; j++)
	   	dv[i][j] = bv[(i-smi)%sbimax][j-smj];

    	for (j = smj-1; j >= 0; j--)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][(j-smj+sbjmax)%sbjmax];
     	for (j = smj+sbjmax; j < sjmax; j++)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][(j-smj)%sbjmax];

      	for (i = 0; i < sbimax; i++)
      	    for (j = 0; j < sbjmax; j++)
            {    
              	num_type val = (num_type) 0;
              	for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
            }
  
        return c;
}
nsl_matrix NSLconv_copy(const nsl_matrix& a,const nsl_matrix& b)
{
      	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
      	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int smi = saimax/2;
	int smj = sajmax/2;

 	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

      	nsl_matrix c(sbimax,sbjmax);
      	nsl_matrix d(simax,sjmax);

      	d.put_sector(b,saimax/2,sajmax/2); 

	int i,j;

	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

      	for (i = smi-1; i >= 0; i--)
    	   for (j = smj; j < sbjmax+smj; j++)
 	   	dv[i][j] = bv[0][j-smj];
      	for (i = smi+sbimax; i < simax; i++)
    	   for (j = smj; j < sbjmax+smj; j++)
	   	dv[i][j] = bv[sbimax-1][j-smj];

    	for (j = smj-1; j >= 0; j--)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][0];
     	for (j = smj+sbjmax; j < sjmax; j++)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][sbjmax-1];

      	for (i = 0; i < sbimax; i++)
      	    for (j = 0; j < sbjmax; j++)
            {    
              	num_type val = (num_type) 0;
              	for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
            }
  
        return c;
}
