/* SCCS  @(#)nsl_vector_conv_library.C	1.1---95/08/13--20:48:20 */
//
//	nsl_vector_conv_library.C
//

# include "kallincl.h"

nsl_vector operator*(const nsl_vector& a,const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator*\n";
	cout << "// (vector) " << a.get_name() << " * (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int saimax = a.get_imax();
	int sbimax = b.get_imax();

	int simax = saimax + sbimax - 1;

	nsl_vector c(sbimax);
	nsl_vector d(simax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();
	num_type *Vd = d.get_vector();

	d.put_sector(b,saimax/2); 

	for (int i = 0; i < sbimax; i++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
			val = val + Va[m] * Vd[i+m];
		Vc[i] = val;
	   }

	return c;
}
nsl_vector operator*(const nsl_vector& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator*\n";
	cout << "// (vector) " << a.get_name() << " * (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < aimax; i++)
		Vc[i] = Va[i] * b;

	return c;
}
nsl_vector operator*(const num_type a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator*\n";
	cout << "// (num_typer) " << a << " * (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);
	num_type *Vb = b.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < bimax; i++)
		Vc[i] = a * Vb[i];

	return c;
}
nsl_vector operator*(const nsl_vector& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator*\n";
	cout << "// (vector) " << a.get_name() << " * (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();
	num_type  Vb = b.get_data();

	for (int i = 0; i < aimax; i++)
		Vc[i] = Va[i] * Vb;

	return c;
}
nsl_vector operator*(const nsl_data& a,const  nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator*\n";
	cout << "// (data) " << a.get_name() << " * (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);
	num_type *Vb = b.get_vector();
	num_type *Vc = c.get_vector();
	num_type  Va = a.get_data();

	for (int i = 0; i < bimax; i++)
		Vc[i] = Va * Vb[i];

	return c;
}

// zero-edge vector convolution

nsl_vector NSLconv_zero(const nsl_vector& a,const nsl_vector& b)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();

	int simax = saimax + sbimax - 1;

	nsl_vector c(sbimax);
	nsl_vector d(simax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

	d.put_sector(b,saimax/2); 

	for (int i = 0; i < sbimax; i++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
			val = val + av[m] * dv[i+m];
		cv[i] = val;
	   }
	return c;
}
nsl_vector NSLconv_zero(const nsl_vector& a,const num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * b;

	return c;
}
nsl_vector NSLconv_zero(const num_type a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = a * bv[i];

	return c;
}
nsl_vector NSLconv_zero(const nsl_vector& a,const nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * bv;

	return c;
}
nsl_vector NSLconv_zero(const nsl_data& a,const  nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type av = a.get_data();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = av * bv[i];

	return c;
}

// wrap around convolution

nsl_vector NSLconv_wrap(const nsl_vector& a,const nsl_vector& b)
{
      	int saimax = a.get_imax();
      	int sbimax = b.get_imax();

	int sm = saimax/2;

      	int simax = saimax + sbimax - 1;

      	nsl_vector c(sbimax);
      	nsl_vector d(simax);

      	d.put_sector(b,saimax/2); 

	int i,m;

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

      	for (i = sm-1; i >= 0; i--)
	   dv[i] = bv[(i-sm+sbimax)%sbimax];
      	for (i = sm+sbimax; i < simax; i++)
	   dv[i] = bv[(i-sm)%sbimax];

     	for (i = 0; i < sbimax; i++)
        {    
              num_type val = (num_type) 0;
              for (m = 0; m < saimax; m++)
                      val = val + av[m] * dv[i+m];
              cv[i] = val;
        }

        return c;
}

// copy boundary convolution

nsl_vector NSLconv_copy(const nsl_vector& a,const nsl_vector& b)
{
      	int saimax = a.get_imax();
      	int sbimax = b.get_imax();

	int sm = saimax/2;

      	int simax = saimax + sbimax - 1;

      	nsl_vector c(sbimax);
      	nsl_vector d(simax);

      	d.put_sector(b,saimax/2); 

	int i,m;

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* dv = d.get_vector();

      	for (i = sm-1; i >= 0; i--)
	   dv[i] = bv[0];
      	for (i = sm+sbimax; i < simax; i++)
	   dv[i] = bv[sbimax-1];

      	for (i = 0; i < sbimax; i++)
        {    
              num_type val = (num_type) 0;
              for (m = 0; m < saimax; m++)
                      val = val + av[m] * dv[i+m];
              cv[i] = val;
        }

        return c;
}

