/* SCCS  @(#)nsl_vector_sub_library.C	1.1---95/08/13--20:48:23 */
//
//	nsl_vector_sub_library.C
//

# include "kallincl.h"

// vector subtraction

nsl_vector operator-(const nsl_vector& a)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (vector) "<< " = - (vector) ";
	cout << a.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < aimax; i++)
		Vc[i] = -Va[i];

	return c;
}
nsl_vector operator-(const nsl_vector& a,const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (vector) " << a.get_name() << " - (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator- ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	num_type *Va = a.get_vector();
	num_type *Vb = b.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < aimax; i++)
		Vc[i] = Va[i] - Vb[i];
	return c;
}
nsl_vector operator-(const nsl_vector& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (vector) " << a.get_name() << " - (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < aimax; i++)
		Vc[i] = Va[i] - b;

	return c;
}
nsl_vector operator-(const num_type a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (num_type) " << a << " - (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);
	num_type *Vb = b.get_vector();
	num_type *Vc = c.get_vector();

	for (int i = 0; i < bimax; i++)
		Vc[i] = a - Vb[i];

	return c;
}
nsl_vector operator-(const nsl_vector& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (vector) " << a.get_name() << " - (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);
	num_type *Va = a.get_vector();
	num_type *Vc = c.get_vector();
	num_type  Vb = b.get_data();

	for (int i = 0; i < aimax; i++)
		Vc[i] = Va[i] - Vb;

	return c;
}
nsl_vector operator-(const nsl_data& a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator-\n";
	cout << "// (data) " << a.get_name() << " - (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);
	num_type *Vb = b.get_vector();
	num_type *Vc = c.get_vector();
	num_type  Va = a.get_data();

	for (int i = 0; i < bimax; i++)
		Vc[i] = Va - Vb[i];

	return c;
}

// vector subtraction

nsl_vector NSLsub(const nsl_vector& a)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = -av[i];

	return c;
}
nsl_vector NSLsub(const nsl_vector& a,const nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::NSLsub ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] + bv[i];

	return c;
}
nsl_vector NSLsub(const nsl_vector& a,const num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] - b;

	return c;
}
nsl_vector NSLsub(const num_type a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* cv = c.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = a - bv[i];

	return c;
}
nsl_vector NSLsub(const nsl_vector& a,const nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] - bv;

	return c;
}
nsl_vector NSLsub(const nsl_data& a,const  nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* cv = c.get_vector();
	num_type* bv = b.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < bimax; i++)
		cv[i] = av - bv[i];

	return c;
}

