/* SCCS  @(#)nsl_matrix_sat_library.C	1.1---95/08/13--20:48:17 */
//
//	nsl_matrix_sat_library.C
//

# include "kallincl.h"

nsl_matrix NSLsat(const nsl_matrix& a)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > (num_type) 1)
			cv[i][j] = (num_type) 1;
		else if (av[i][j] > 0)
			cv[i][j] = av[i][j];
		else
			cv[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_matrix& x1,const nsl_matrix& x2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax()||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** x1v = x1.get_matrix();
	num_type** x2v = x2.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v[i][j])
		    cv[i][j] = 1.0;
		else if (av[i][j] > x1v[i][j])
		    cv[i][j] = (av[i][j]-x1v[i][j])/(x2v[i][j]-x1v[i][j]);
		else
		    cv[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const num_type x1,const num_type x2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2)
			cv[i][j] = 1.0;
		else if (av[i][j] > x1)
			cv[i][j] = (av[i][j]-x1)/(x2-x1);
		else
			cv[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_data& x1,const nsl_data& x2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v)
			cv[i][j] = 1.0;
		else if (av[i][j] > x1v)
			cv[i][j] = (av[i][j]-x1v)/(x2v-x1v);
		else
			cv[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_matrix& x1,const nsl_matrix& x2,
	const nsl_matrix& y1,const nsl_matrix& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax() ||
	    a.get_imax() != y1.get_imax() || a.get_jmax() != y1.get_jmax() ||
	    a.get_imax() != y2.get_imax() || a.get_jmax() != y2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error("imax4: ",y1.get_imax());
		cmd_error("jmax4: ",y1.get_jmax());
		cmd_error("imax5: ",y2.get_imax());
		cmd_error("jmax5: ",y2.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** x1v = x1.get_matrix();
	num_type** x2v = x2.get_matrix();
	num_type** y1v = y1.get_matrix();
	num_type** y2v = y2.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v[i][j])
		    cv[i][j] = y2v[i][j];
		else if (av[i][j] > x1v[i][j])
		    cv[i][j] = (y2v[i][j]-y1v[i][j])*(av[i][j]-x1v[i][j])/
				(x2v[i][j]-x1v[i][j]) + y1v[i][j];
		else
		    cv[i][j] = y1v[i][j];
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_matrix& x1,const nsl_matrix& x2,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** x1v = x1.get_matrix();
	num_type** x2v = x2.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v[i][j])
		    cv[i][j] = y2;
		else if (av[i][j] > x1v[i][j])
		    cv[i][j] = (y2-y1)*(av[i][j]-x1v[i][j])/
				(x2v[i][j]-x1v[i][j]) + y1;
		else
		    cv[i][j] = y1;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_matrix& x1,const nsl_matrix& x2,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);

	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error("bad matrix sector");
		return c;
	}

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** x1v = x1.get_matrix();
	num_type** x2v = x2.get_matrix();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v[i][j])
		    cv[i][j] = y2v;
		else if (av[i][j] > x1v[i][j])
		    cv[i][j] = (y2v-y1v)*(av[i][j]-x1v[i][j])/
				(x2v[i][j]-x1v[i][j]) + y1v;
		else
		    cv[i][j] = y1v;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const num_type x1,const num_type x2,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2)
		    cv[i][j] = y2;
		else if (av[i][j] > x1)
		    cv[i][j] = (y2-y1)*(av[i][j]-x1)/(x2-x1) + y1;
		else
		    cv[i][j] = y1;
	return c;
}
nsl_matrix NSLsat(const nsl_matrix& a,const nsl_data& x1,const nsl_data& x2,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (av[i][j] > x2v)
		    cv[i][j] = y2v;
		else if (av[i][j] > x1v)
		    cv[i][j] = (y2v-y1v)*(av[i][j]-x1v)/(x2v-x1v) + y1v;
		else
		    cv[i][j] = y1v;
	return c;
}
