/* SCCS  @(#)nsl_vector_sat_library.C	1.1---95/08/13--20:48:24 */
//
//	nsl_vector_sat_library.C
//

# include "kallincl.h"

nsl_vector NSLsat(const nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > (num_type) 1)
			cv[i] = (num_type) 1;
		else if (av[i] > 0)
			cv[i] = av[i];
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_vector& x1,const nsl_vector& x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
			cv[i] = (num_type) 1;
		else if (av[i] > x1v[i])
			cv[i] = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const num_type x1,const num_type x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2)
			cv[i] = (num_type) 1;
		else if (av[i] > x1)
			cv[i] = (av[i] - x1)/(x2 - x1);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_data& x1,const nsl_data& x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v)
			cv[i] = (num_type) 1;
		else if (av[i] > x1v)
			cv[i] = (av[i] - x1v)/(x2v - x1v);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_vector& x1,const nsl_vector& x2,
	const nsl_vector& y1,const nsl_vector& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != x1.get_imax() ||  a.get_imax() != x2.get_imax() || 
	    a.get_imax() != y1.get_imax() ||  a.get_imax() != y2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("imax4: ",y1.get_imax());
		cmd_error("imax5: ",y2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2v[i];
		else if (av[i] > x1v[i])
		    cv[i] = (y2v[i]-y1v[i])*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1v[i];
		else
		    cv[i] = y1v[i];
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_vector& x1,const nsl_vector& x2,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2;
		else if (av[i] > x1v[i])
		    cv[i] = (y2-y1)*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1;
		else
		    cv[i] = y1;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_vector& x1,const nsl_vector& x2,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2v;
		else if (av[i] > x1v[i])
		    cv[i] = (y2v-y1v)*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1v;
		else
		    cv[i] = y1v;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const num_type x1,const num_type x2,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2)
		    cv[i] = y2;
		else if (av[i] > x1)
		    cv[i] = (y2-y1)*(av[i]-x1)/(x2-x1) + y1;
		else
		    cv[i] = y1;
	return c;
}
nsl_vector NSLsat(const nsl_vector& a,const nsl_data& x1,const nsl_data& x2,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v)
		    cv[i] = y2v;
		else if (av[i] > x1v)
		    cv[i] = (y2v-y1v)*(av[i]-x1v)/(x2v-x1v) + y1v;
		else
		    cv[i] = y1v;
	return c;
}
