/* SCCS  @(#)nsl_vector_step_library.C	1.1---95/08/13--20:48:24 */
//
//	nsl_vector_step_library.C
//

# include "kallincl.h"

// if a > 0 then 1 else 0

nsl_vector NSLstep(const nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > (num_type) 0)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}

// if a > b then 1 else 0

nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}

// if a > b then y2 else y1

nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const nsl_vector& y1,const nsl_vector& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2v[i];
		else
			cv[i] = y1v[i];
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_vector& b,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("bad vector sector");
		return c;
	}

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2v;
		else
			cv[i] = y1v;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const num_type b,
	const num_type y1,const num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLstep(const nsl_vector& a,const nsl_data& b,
	const nsl_data& y1,const nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = y2v;
		else
			cv[i] = y1v;
	return c;
}
