//
//	nsl_num_library.C
//

# include "nsl_all_kernel_include.h"

// mult_equal

// data

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() = a.get_data() * b;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	out.elem() = 0;
	for (int i = 0; i < amax; i++)
		out.elem() += av[i] * bv;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	out.elem() = 0;
	for (int i = 0; i < amax; i++)
		out.elem() += av[i] * bv[i];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	num_type** av = a.get_matrix();
	num_type bv = b.get_data();

	out.elem() = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out.elem() += av[i][j] * bv;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();

	out.elem() = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out.elem() += av[i][j] * bv[i][j];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() = a.get_data() * b.get_data();
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	out.elem() = 0;
	for (int i = 0; i < bmax; i++)
		out.elem() += av * bv[i];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	num_type av = a.get_data();
	num_type** bv = b.get_matrix();

	out.elem() = 0;
	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		out.elem() += av * bv[i][j];
}

// vector

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] = av[i] * b;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] = av[i] * bv;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();
	int amax = a.get_imax();
	int bmax = b.get_imax();

	if (amax != bmax || bmax != outmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] = av[i] * bv[i];
}

// mat to vec

void NSLmult_equal(nsl_num_vector& out,const nsl_num_matrix& b,
	const nsl_num_data& a)
{
	int simax = out.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	num_type  *Vc = out.get_vector();
	num_type **Vb = b.get_matrix();
	num_type  Va = a.get_data();

	out = 0;

	if (out.get_vec_type() == 0) // ROW
	{
		if (simax != sbjmax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
			Vc[j] += Va*Vb[i][j];
	}
	else // COL
	{
		if (simax != sbimax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < sbimax; i++)
		    for (int j = 0; j < sbjmax; j++)
			Vc[i] += Va*Vb[i][j];
	}
}
void NSLmult_equal(nsl_num_vector& out,const nsl_num_matrix& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bmax = b.get_imax();

	if (aimax != bmax || bmax != outmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type* outv = out.get_vector();
	num_type** av = a.get_matrix();
	num_type* bv = b.get_vector();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		outv[i] = av[i][j] * bv[i];
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] = av * bv;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] = av * b;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] = av * bv[i];
}

// matrix

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av[i][j] * b;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av[i][j] * bv;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av[i][j] * bv[i][j];
}

// matrix conv
// fix for no allocation/deallocation of d

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& b,
	const nsl_num_vector& a)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	out = 0; // needed

	num_type **Vc = out.get_matrix();
	num_type  *Va = a.get_vector();

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		num_type **Vd = d.get_matrix();

		d.put_sector(b,0,saimax/2); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i][j+m];
			Vc[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		num_type **Vd = d.get_matrix();

		d.put_sector(b,saimax/2,0); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + Va[m] * Vd[i+m][j];
			Vc[i][j] = val;
		   }
	}
}

// vec to mat

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_vector& b,
	const nsl_num_data& a)
{
	int sbimax = b.get_imax();
	int simax = out.get_imax();
	int sjmax = out.get_jmax();

	num_type **Vc = out.get_matrix();
	num_type  *Vb = b.get_vector();
	num_type  Va = a.get_data();

	if (b.get_vec_type() == 0) // ROW
	{
		if (sbimax != sjmax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < simax; i++)
		   for (int j = 0; j < sjmax; j++)
			Vc[i][j] = Va*Vb[j];
	}
	else // COL
	{
		if (sbimax != simax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < simax; i++)
		    for (int j = 0; j < sjmax; j++)
			Vc[i][j] = Va*Vb[i];
	}
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av * b;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av * bv;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av * bv[i][j];
}

// mult_plus

// data

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() += a.get_data() * b;
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < amax; i++)
		out.elem() += av[i] * bv;
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < amax; i++)
		out.elem() += av[i] * bv[i];
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() += a.get_data() * b.get_data();
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	for (int i = 0; i < bmax; i++)
		out.elem() += av * bv[i];
}

// vector

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] += av[i] * b;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] += av[i] * bv;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] += av[i] * bv[i];
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] += av * b;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] += av * bv;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] += av * bv[i];
}

// matrix

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av[i][j] * b;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av[i][j] * bv;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av[i][j] * bv[i][j];
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av * b;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av * bv;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av * bv[i][j];
}

// mult_minus

// data

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() -= a.get_data() * b;
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < amax; i++)
		out.elem() -= av[i] * bv;
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < amax; i++)
		out.elem() -= av[i] * bv[i];
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() -= a.get_data() * b.get_data();
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	for (int i = 0; i < bmax; i++)
		out.elem() -= av * bv[i];
}

// vector

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av[i] * b;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av[i] * bv;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av[i] * bv[i];
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av * b;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av * bv;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();
	num_type* bv = b.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av * bv[i];
}

// matrix

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av[i][j] * b;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av[i][j] * bv;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av[i][j] * bv[i][j];
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av * b;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type bv = b.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av * bv;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();
	num_type** bv = b.get_matrix();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av * bv[i][j];
}

// add_equal

// data

void NSLadd_equal(nsl_num_data& out,const nsl_num_matrix& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	num_type** av = a.get_matrix();

	out.elem() = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out.elem() += av[i][j];
}
void NSLadd_equal(nsl_num_data& out,const nsl_num_vector& a)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();

	out.elem() = 0;
	for (int i = 0; i < amax; i++)
		out.elem() += av[i];
}
void NSLadd_equal(nsl_num_data& out,const nsl_num_data& a)
{
	out = a;
}

// vector

void NSLadd_equal(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] = av[i];
}
void NSLadd_equal(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] = av;
}

// mat_to_vec

void NSLadd_equal(nsl_num_vector& out,const nsl_num_matrix& a)
{
	int vec_type = out.get_vec_type();

	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	int s = out.get_imax();

	num_type* outv = out.get_vector();
	num_type** av = a.get_matrix();

	out = 0;
	
	if (vec_type == 0) // row vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    outv[j] += av[i][j];
	}
	if (vec_type == 1) // col vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    outv[i] += av[i][j];
	}
}

// matrix

void NSLadd_equal(nsl_num_matrix& out,const nsl_num_matrix& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLadd_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		outv[i][j] = av[i][j];
}
void NSLadd_equal(nsl_num_matrix& out,const nsl_num_data& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] = av;
}

// vec_to_mat

void NSLadd_equal(nsl_num_matrix& out,const nsl_num_vector& a)
{
	int vec_type = a.get_vec_type();

	int simax = out.get_imax();
	int sjmax = out.get_jmax();

	int s = a.get_imax();

//	out = 0;

	num_type* av = a.get_vector();
	num_type** outv = out.get_matrix();
	
	if (vec_type == 0) // row vector
	{
	    if (s != sjmax)
	    {
		cmd_error("*** NSLvec_to_mat ***");
		cmd_error("vector jmax: ",s);
		cmd_error("matrix jmax: ",sjmax);
		cmd_error("Bad vector size");
		return;
	    }

	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    outv[i][j] = av[j];
	}
	if (vec_type == 1) // col vector
	{
	    if (s != simax)
	    {
		cmd_error("*** NSLvec_to_mat ***");
		cmd_error("vector imax: ",s);
		cmd_error("matrix imax: ",simax);
		cmd_error("Bad vector size");
		return;
	    }

	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    outv[i][j] = av[i];
	}
}

// add_plus

// data

void NSLadd_plus(nsl_num_data& out,const nsl_num_matrix& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	num_type** av = a.get_matrix();

	out.elem() = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out.elem() += av[i][j];
}
void NSLadd_plus(nsl_num_data& out,const nsl_num_vector& a)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();

	for (int i = 0; i < amax; i++)
		out.elem() += av[i];
}
void NSLadd_plus(nsl_num_data& out,const nsl_num_data& a)
{
	out.elem() += a.get_data();
}

// vector

void NSLadd_plus(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] += av[i];
}
void NSLadd_plus(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] += av;
}

// matrix

void NSLadd_plus(nsl_num_matrix& out,const nsl_num_matrix& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLadd_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		outv[i][j] += av[i][j];
}
void NSLadd_plus(nsl_num_matrix& out,const nsl_num_data& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] += av;
}

// add_minus

// data

void NSLadd_minus(nsl_num_data& out,const nsl_num_matrix& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	num_type** av = a.get_matrix();

	out.elem() = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out.elem() -= av[i][j];
}
void NSLadd_minus(nsl_num_data& out,const nsl_num_vector& a)
{
	int amax = a.get_imax();

	num_type* av = a.get_vector();

	for (int i = 0; i < amax; i++)
		out.elem() -= av[i];
}
void NSLadd_minus(nsl_num_data& out,const nsl_num_data& a)
{
	out.elem() -= a.get_data();
}

// vector

void NSLadd_minus(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av[i];
}
void NSLadd_minus(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	num_type* outv = out.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < outmax; i++)
		outv[i] -= av;
}

// matrix

void NSLadd_minus(nsl_num_matrix& out,const nsl_num_matrix& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLadd_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	num_type** outv = out.get_matrix();
	num_type** av = a.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		outv[i][j] -= av[i][j];
}
void NSLadd_minus(nsl_num_matrix& out,const nsl_num_data& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	num_type** outv = out.get_matrix();
	num_type av = a.get_data();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		outv[i][j] -= av;
}
