//
// 	nsl_model.h
//

class nsl_model
{
	char name[NAME_SIZE];
	int index;

	float DELTA;		// time step

	float CURRENT_TIME;	// current time
	int CURRENT_TIME_STEP;

	float END_TIME;		// end time
	int END_TIME_STEP;

	int able_fg; 		// '1' - enable model, '0' - disable
	int verbose_fg;		// '1' - run verbose 
	int print_time_fg;	// '1' - print time 

	int layer_total; 	// total number of layers
	layer_obj_list* nsl_layer_list; // pointer to all nsl layers

	int module_total; 	// total number of run modules
	module_type_list* nsl_module_list; // pointer to all nsl modules

public:
		nsl_model(char*);
		nsl_model();

	void set_name(char* str) { strcpy(name,str); }
	void set_index(int i) { index = i; }
	char* get_name() { return name; }
	int get_index()	{ return index; }

// get time var
	float get_delta() { return DELTA; }
	float get_time() { return CURRENT_TIME; }
	int get_time_step() { return CURRENT_TIME_STEP; }
	float get_end_time() { return END_TIME; }
	int get_end_time_step() { return END_TIME_STEP; }

// set time var
	void 	set_delta(float);
	void 	set_time(float);
	void 	set_time(int); 
	void 	set_end_time(float);
	void 	set_end_time(int);

// set flags
	void 	set_print_time_fg(int i) { print_time_fg = i; }
	void 	set_able_fg(int i) { able_fg = i; }
	void 	set_verbose_fg(int i) { verbose_fg = i; }
	void 	set_all_verbose_fg(int i);
// get flags
	int 	get_able_fg() { return able_fg; }
	int 	get_print_time_fg() { return print_time_fg; }
	int 	get_verbose_fg() { return verbose_fg; }

// enable, disable
	void 	enable() { able_fg = 1; }
	void 	disable() { able_fg = 0; }

// reset time
	void reset_time();
// increment time 
	void incr_time();

// layers
	int 	get_layer_total() { return layer_total; }
	layer_obj_list* get_layer_list() { return nsl_layer_list; }

	int	add_layer(nsl_data*);
	int	add_layer(nsl_vector*);
	int	add_layer(nsl_matrix*);

	nsl_layer*	get_layer();		// get first layer
	nsl_layer*	get_layer(char*);	// name search
	nsl_layer*	get_layer(int);		// index search

// input layers
	nsl_layer*	get_input_layer();	// get first layer
	nsl_layer*	get_input_layer(char*);	// name search
	nsl_layer*	get_input_layer(int);	// index search

// modules
	int get_module_total() { return module_total; }
	module_type_list* get_module_list() { return nsl_module_list; }

	int	add_module(nsl_module*);

	nsl_module*	get_module(char*);	// name search
	nsl_module*	get_module(int);	// index search

	void run();

// print
friend	ostream& 	operator<<(ostream&,nsl_model&);
friend	istream& 	operator>>(istream&,nsl_model&);
friend	ostream& 	operator<<(ostream&,nsl_model*);
friend	istream& 	operator>>(istream&,nsl_model*);

// header for output data file
	void file_header(ostream&);

// enable module
	void enable_all_module();
	void enable_module(char*);
	void enable_module(int);

// disable modules
	void disable_all_module();
	void disable_module(char*);
	void disable_module(int);

// enable layer
	void enable_all_layer();
	void enable_layer(char*);
	void enable_layer(int);
// disable layer
	void disable_all_layer();
	void disable_layer(char*);
	void disable_layer(int);

// **** status ****

// print model status
	void print_status(ostream&);
// print module status
	void print_all_module_status(ostream&);
	void print_module_status(ostream&,char*);
	void print_module_status(ostream&,int);
// print layer status
	void print_all_layer_status(ostream&);
	void print_layer_status(ostream&,char*);
	void print_layer_status(ostream&,int);
// print layer status
	void print_all_input_layer_status(ostream&);
	void print_input_layer_status(ostream&,char*);
	void print_input_layer_status(ostream&,int);
// print layer data
	void print_all_layer_data(ostream&);
	void print_layer_data(ostream&,char*);
	void print_layer_data(ostream&,int);
// print input layer status
	void print_all_input_layer_stim(ostream&);
	void print_input_layer_stim(ostream&,char*);
	void print_input_layer_stim(ostream&,int);
};

typedef nsl_model nsl_network;

// model list 
typedef nsl_model* model_type;
declare(gslist,model_type);

typedef gslist_iterator(model_type) model_type_iterator;
typedef gslist(model_type) model_type_list;
