//
//	nsl_vector.h
//

extern class nsl_matrix;
extern class nsl_model;

struct nsl_vector : public nsl_layer
{
	num_type* v;
	int i0,i1;
	int imax;

	int vec_type;	// 0 - row vector, 1 - col vector
public:
// creation without memory allocation
		nsl_vector();
// creation with memory allocation
		nsl_vector(int);
		nsl_vector(nsl_vector&);
// creation and addition of layer to nsl_model
		nsl_vector(nsl_model*,char*,int);
		nsl_vector(nsl_model&,char*,int);

		nsl_vector(char*,int);
		nsl_vector(char*,int,int); // last par is vec_type

		~nsl_vector();

	// alloc memory
	int	alloc_mem(int);

	// transpose vector
	void 	set_vec_type(int i) { vec_type = i; }
	int	get_vec_type() { return vec_type; }

	// old get
	int	get_x0() { return i0; }
	int	get_x1() { return i1; }
	int	get_xn() { return imax; }
	// get
	int	get_i0() { return i0; }
	int	get_i1() { return i1; }
	int	get_imax() { return imax; }
	int	get_size() { return imax; }
	num_type*	get_vector() { return v; }

// element retrieval
num_type&	operator[](int);
num_type&	elem(int i);

// assignment
void 	operator=(nsl_vector&);
void 	operator=(nsl_data&);
void 	operator=(num_type);
// void 	operator=(num_type*);

// arithmetic

// addition
friend 	nsl_vector 	operator+(nsl_vector&);
friend 	nsl_vector 	operator+(nsl_vector&,nsl_vector&);
friend 	nsl_vector 	operator+(nsl_vector&,num_type);
friend 	nsl_vector 	operator+(num_type,nsl_vector&);
friend 	nsl_vector 	operator+(nsl_vector&,nsl_data&);
friend 	nsl_vector 	operator+(nsl_data&,nsl_vector&);

// subtraction
friend 	nsl_vector 	operator-(nsl_vector&);
friend 	nsl_vector 	operator-(nsl_vector&,nsl_vector&);
friend	nsl_vector 	operator-(nsl_vector&,num_type);
friend	nsl_vector 	operator-(num_type,nsl_vector&);
friend 	nsl_vector 	operator-(nsl_vector&,nsl_data&);
friend 	nsl_vector 	operator-(nsl_data&,nsl_vector&);

// pointwise multiplication
friend 	nsl_vector 	operator^(nsl_vector&,nsl_vector&);
friend	nsl_vector 	operator^(nsl_vector&,num_type);
friend	nsl_vector 	operator^(num_type,nsl_vector&);
friend	nsl_vector 	operator^(nsl_vector&,nsl_data&);
friend 	nsl_vector 	operator^(nsl_data&,nsl_vector&);

// pointwise division
friend 	nsl_vector 	operator/(nsl_vector&,nsl_vector&);
friend	nsl_vector 	operator/(nsl_vector&,num_type);
friend	nsl_vector 	operator/(num_type,nsl_vector&);
friend 	nsl_vector 	operator/(nsl_vector&,nsl_data&);
friend 	nsl_vector 	operator/(nsl_data&,nsl_vector&);

// convolution
friend 	nsl_vector 	operator*(nsl_vector&,nsl_vector&);
friend	nsl_matrix 	operator*(nsl_vector&,nsl_matrix&);
friend	nsl_matrix 	operator*(nsl_matrix&,nsl_vector&);
friend 	nsl_vector 	operator*(nsl_vector&,nsl_data&);
friend 	nsl_vector 	operator*(nsl_data&,nsl_vector&);
friend	nsl_vector 	operator*(nsl_vector&,num_type);
friend	nsl_vector 	operator*(num_type,nsl_vector&);

//	threshold functions

// NSLstep
friend	nsl_vector 	NSLstep(nsl_vector&);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLstep(nsl_vector&,num_type);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_data&);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_vector&,nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_vector&,num_type,num_type);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_vector&,nsl_data&,nsl_data&);
friend	nsl_vector 	NSLstep(nsl_vector&,num_type,num_type,num_type);
friend	nsl_vector 	NSLstep(nsl_vector&,nsl_data&,nsl_data&,nsl_data&);

// NSLramp
friend	nsl_vector 	NSLramp(nsl_vector&);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLramp(nsl_vector&,num_type);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_data&);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_vector&,nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_vector&,num_type,num_type);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_vector&,nsl_data&,nsl_data&);
friend	nsl_vector 	NSLramp(nsl_vector&,num_type,num_type,num_type);
friend	nsl_vector 	NSLramp(nsl_vector&,nsl_data&,nsl_data&,nsl_data&);

// NSLsat
friend	nsl_vector 	NSLsat(nsl_vector&);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLsat(nsl_vector&,num_type,num_type);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_data&,nsl_data&);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_vector&,nsl_vector&,
				nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_vector&,nsl_vector&,
				num_type,num_type);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_vector&,nsl_vector&,
				nsl_data&,nsl_data&);
friend	nsl_vector 	NSLsat(nsl_vector&,num_type,num_type,
				num_type,num_type);
friend	nsl_vector 	NSLsat(nsl_vector&,nsl_data&,nsl_data&,
				nsl_data&,nsl_data&);

// NSLsigma
friend	nsl_vector 	NSLsigma(nsl_vector&);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLsigma(nsl_vector&,num_type,num_type);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_data&,nsl_data&);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_vector&,nsl_vector&,
				nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_vector&,nsl_vector&,
				num_type,num_type);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_vector&,nsl_vector&,
				nsl_data&,nsl_data&);
friend	nsl_vector 	NSLsigma(nsl_vector&,num_type,num_type,
				num_type,num_type);
friend	nsl_vector 	NSLsigma(nsl_vector&,nsl_data&,nsl_data&,
				nsl_data&,nsl_data&);

// 	max,min functions

// NSLmax
	num_type	max();
friend	num_type 	NSLmax(nsl_vector&);
friend	nsl_vector 	NSLmax(nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLmax(nsl_vector&,nsl_data&);
friend	nsl_vector 	NSLmax(nsl_vector&,num_type);

// NSLmin
	num_type	min();
friend	num_type 	NSLmin(nsl_vector&);
friend	nsl_vector 	NSLmin(nsl_vector&,nsl_vector&);
friend	nsl_vector 	NSLmin(nsl_vector&,nsl_data&);
friend	nsl_vector 	NSLmin(nsl_vector&,num_type);

// misc
friend	nsl_vector 	NSLmat_to_vec(nsl_vector&,nsl_matrix&);
friend	nsl_vector 	NSLmat_to_col_vec(nsl_vector&,nsl_matrix&);
friend	nsl_vector 	NSLmat_to_row_vec(nsl_vector&,nsl_matrix&);
friend	nsl_matrix 	NSLvec_to_mat(nsl_matrix&,nsl_vector&);
friend	nsl_matrix 	NSLcol_vec_to_mat(nsl_matrix&,nsl_vector&);
friend	nsl_matrix 	NSLrow_vec_to_mat(nsl_matrix&,nsl_vector&);

	nsl_matrix expand_col(int);
	nsl_matrix expand_row(int);
	nsl_matrix expand(int);

	nsl_vector transpose_col();
	nsl_vector transpose_row();
	nsl_vector transpose();

	// sector
	nsl_vector 	get_sector(int,int);
	nsl_vector 	set_sector(nsl_vector&,int);
	nsl_vector 	put_sector(nsl_vector&,int); // same as set_sector

	// sum
	num_type	sum();

	// print
friend	ostream& 	operator<<(ostream&,nsl_vector&);
friend	istream& 	operator>>(istream&,nsl_vector&);

// print data
		void	print(ostream&);
		void	print(ostream&,int,int);
// print status
	virtual void	print_status(ostream&); // virtual
		void	print_layer_status(ostream&); // local

	virtual	void	print_stim(ostream&) {}
	virtual	void	print_stim(ostream&,int) {}

	virtual nsl_stim_obj* get_stim(int) { return (nsl_stim_obj*) 0; }
	virtual nsl_stim_obj* get_stim(char*) { return (nsl_stim_obj*) 0; }
};

