# include <stdio.h>
# include <math.h>
# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;



/*    Convolution used in the 140x140 Retina  */

nsl_matrix newconv(nsl_matrix& a,nsl_matrix& b)
{
	int sbimax = b.get_imax();
        int sbjmax = b.get_jmax();
	nsl_matrix c(sbjmax,sbjmax);
	
	c = a*b;

return c;
}



/* 
nsl_matrix newconv(nsl_matrix& a,nsl_matrix& b)
// a is the Mask and b is the input layer.
{
         int saimax = a.get_imax();
         int sajmax = a.get_jmax();
         int sbimax = b.get_imax();
         int sbjmax = b.get_jmax();

        int leftbound = 60;       // 32; for the 72x72; 100 for 140x140.
        nsl_matrix c(30,30); // Make this variable size.
			   // c(8,8) for 72x72; c(25,25) for 140x140.

        for (int i = 0; i < leftbound; i = i+2)
	{

           for (int j = 0; j < leftbound; j = j+2)
           {    
                num_type val = (num_type) 0;

                for (int m = 0; m < saimax; m++)
                   for (int n = 0; n < sajmax; n++)
                        val = val + a.elem(m,n) * b.elem(i+m,j+n);
                c.elem(i/2,j/2) = val;
			
           }
	}
        return c;
}

*/


/* Convolution used from the retina to the thalamus */

nsl_matrix conv25_1(nsl_matrix& a,nsl_matrix& b)
// a is the Mask and b is the input layer.
{
         int saimax = a.get_imax();
         int sajmax = a.get_jmax();
         int sbimax = b.get_imax();
         int sbjmax = b.get_jmax();

        int leftbound = 1;       
        nsl_matrix c(1,1); // Make this variable size.
			   

        for (int i = 0; i < leftbound; i++)
	{

           for (int j = 0; j < leftbound; j++)
           {    
                num_type val = (num_type) 0;

                for (int m = 0; m < saimax; m++)
                   for (int n = 0; n < sajmax; n++)
                        val = val + a.elem(m,n) * b.elem(i+m,j+n);
                c.elem(i,j) = val;
			
           }
	}
        return c;
}


/*    Convolution used for different Masks for every neuron  */

nsl_matrix mult_conv(nsl_matrix& a,nsl_matrix& b)
// a is the Mask and b is the input layer.
{
         int saimax = a.get_imax();
         int sajmax = a.get_jmax();
         int sbimax = b.get_imax();
         int sbjmax = b.get_jmax();

        int leftbound = 60;       // 32; for the 72x72; 100 for 140x140.
        nsl_matrix c(30,30); // Make this variable size.
			   // c(8,8) for 72x72; c(25,25) for 140x140.

        for (int i = 0; i < leftbound; i = i+2)
	{

           for (int j = 0; j < leftbound; j = j+2)
           {    
                num_type val = (num_type) 0;

                for (int m = 0; m < saimax; m++)
                   for (int n = 0; n < sajmax; n++)
                        val = val + a.elem(m,n) * b.elem(i+m,j+n);
                c.elem(i/2,j/2) = val;
			
           }
	}
        return c;
}


// ___________________________ AVERAGE EXACT METHOD ____________________________

nsl_matrix diff_ae(nsl_matrix& V,nsl_matrix& S,nsl_matrix& Prev, nsl_data& tm)
{

         static float dt,tc,temp;
         int Vmax = V.get_imax();    // Size of the matrices.
         
	 nsl_matrix term1 (Vmax,Vmax);
	 nsl_matrix term2 (Vmax,Vmax);

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** AVERAGE EXACT METHOD ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** AVERAGE EXACT METHOD ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}



/* ********************* This is the real Calculation ********************** */

	tc = tm.get_data(); // time constant.
	temp = exp(-dt/tc);

	term1 = (nsl_matrix) ( (1 - temp) * S + temp * V ) ;
	term2 = (nsl_matrix) ( (S - Prev) * (temp * (tc + dt) - tc) / dt); 
	 return (nsl_matrix) (term1 + term2);



}
// THE FOLLOWING CODE IS NOT REALLY IN USE.


/*

nsl_matrix diff_ae(nsl_matrix& V,nsl_matrix& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix diff_ae(nsl_matrix& V,nsl_data& S,nsl_data& tm)
{
	float dt;

	// printf(" Estoy en --> 2\n");

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix diff_ae(nsl_matrix& V,nsl_data& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix diff_ae(nsl_matrix& V,num_type S,nsl_data& tm)
{
	float dt;

	// printf(" Estoy en --> 3\n");

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix diff_ae(nsl_matrix& V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}

*/
