
/****************************************************************/
/*               Homework assignment #8                       	*/
/*                       2d_dylink.h                          	*/
/*         pattern recognition with dynamical links             */
/*                                                            	*/
/*	   Class definitions and member functions for a class   */
/*	   to store four dimensional matrix and one to store 	*/
/*	   a two dimensional matrix of vectors.			*/
/****************************************************************/


class four_d_matrix{

  private:
	int IN_X, IN_Y, OUT_X, OUT_Y;	// the dimensions of the 4-d matrix
	nsl_matrix  **f_d_array;	//2-d array of nsl_matrices



  public:


// Constructors
	four_d_matrix(int in_x, int in_y, int out_x, int out_y);

	four_d_matrix(int out_x, int out_y);

	four_d_matrix();

//Destructor
	~four_d_matrix()
	{
	   delete [IN_X * IN_Y] f_d_array;
	}
	
//	void set_xsize (int x_size) { IN_X = x_size;}
//	void set_ysize (int y_size) { IN_Y = y_size;}

	float* convert_to_2d ();	//convert to format acceptable to dl2ps

	void operator = (float);	//Equality operator for initialization

	nsl_matrix& elem (int i, int j);	//elem function to return
						//the nsl matrix at (i,j)



};

/****************************************************************/

//Member Functions:

four_d_matrix :: four_d_matrix(int in_x, int in_y, int out_x, int out_y) 
{

   int i, j;
   IN_X = in_x;
   IN_Y = in_y;

   OUT_X = out_x;
   OUT_Y = out_y;

//The 4-d matrix is allocated dynamically

   f_d_array = new (nsl_matrix* [IN_X]);

   for (i = 0; i < IN_X; i++)
	f_d_array[i] = new (nsl_matrix [IN_Y]);

   for ( i = 0; i < IN_X; i++)
     for ( j = 0; j < IN_Y; j++)
	f_d_array[i][j].alloc_mem(out_x, out_y);
}

/****************************************************************/

four_d_matrix :: four_d_matrix(int out_x, int out_y) 
{
   int i, j;
   IN_X = 7;	//default values
   IN_Y = 10;
   OUT_X = out_x;
   OUT_Y = out_y;

   f_d_array = new (nsl_matrix* [IN_X]);

   for (i = 0; i < IN_X; i++)
	f_d_array[i] = new (nsl_matrix [IN_Y]);

   for ( i = 0; i < IN_X; i++)
     for ( j = 0; j < IN_Y; j++)
	f_d_array[i][j].alloc_mem(out_x, out_y);
}

/****************************************************************/

four_d_matrix :: four_d_matrix() 
{
   int i, j;
   IN_X = 7;
   IN_Y = 10;
   OUT_X = 7;
   OUT_Y = 10;

   f_d_array = new (nsl_matrix* [IN_X]);

   for (i = 0; i < IN_X; i++)
	f_d_array[i] = new (nsl_matrix [IN_Y]);

   for ( i = 0; i < IN_X; i++)
     for ( j = 0; j < IN_Y; j++)
	f_d_array[i][j].alloc_mem(OUT_X, OUT_Y);
}

/****************************************************************/

float* four_d_matrix :: convert_to_2d ()
{

  float *temp_array;

  temp_array = new float[IN_X * IN_Y * OUT_X * OUT_Y];

  int in_x, in_y, out_x, out_y;

  for (in_x = 0; in_x < IN_X; in_x++)
    for (in_y = 0; in_y < IN_Y; in_y++)
      for (out_x = 0; out_x < OUT_X; out_x++)
	for (out_y = 0; out_y < OUT_Y; out_y++)
	   temp_array [((in_x * IN_Y) + in_y) * (OUT_X * OUT_Y) + ((out_x * OUT_Y) + out_y)] 
		= f_d_array[in_x][in_y].elem(out_x, out_y);

//convert the 4-d matrix to a linear array which can be used in dl2ps

  return temp_array;


}

/****************************************************************/

void four_d_matrix :: operator = (float number)
{

   for (int i = 0; i < IN_X; i++)
     for (int j = 0; j < IN_Y; j++)
	for (int k = 0; k < OUT_X; k++)
	  for (int l = 0; l < OUT_Y; l++)
		f_d_array[i][j].elem(k, l) = number;

}

/****************************************************************/

nsl_matrix& four_d_matrix :: elem (int i, int j)
{
	return (nsl_matrix&) f_d_array[i][j];
}

/****************************************************************/
/****************************************************************/

//The class jet_graph stores 2-d graphs of jets

class jet_graph{

  private:
	int x_size, y_size, jet_size;
	nsl_vector  **vec_array;	//2-d array of nsl_vectors


  public:

//Constructor
	jet_graph (int x, int y, int vec_size);

	jet_graph (int vec_size);

//Destructor
	~jet_graph()
	{
	   delete [x_size * y_size] vec_array;
	}

	nsl_vector& elem (int i, int j);

	void operator = (float);


};

/****************************************************************/

jet_graph :: jet_graph(int x, int y, int vec_size)
{
  int i, j;

  x_size = x;
  y_size = y;
  jet_size = vec_size;

   vec_array = new (nsl_vector* [x_size]);

   for (i = 0; i < x_size; i++)
	vec_array[i] = new (nsl_vector [y_size]);

  for (i = 0; i < x_size; i++)
    for (j = 0; j < y_size; j++)
	vec_array[i][j].alloc_mem(vec_size);
}

/****************************************************************/

jet_graph :: jet_graph(int vec_size)
{
  int i, j;

  x_size = 7;
  y_size = 10;
  jet_size = vec_size;

   vec_array = new (nsl_vector* [x_size]);

   for (i = 0; i < x_size; i++)
	vec_array[i] = new (nsl_vector [y_size]);

  for (i = 0; i < x_size; i++)
    for (j = 0; j < y_size; j++)
	vec_array[i][j].alloc_mem(vec_size);
}

/****************************************************************/

nsl_vector& jet_graph :: elem (int i, int j)
{

	return (nsl_vector&) vec_array[i][j];
}

/****************************************************************/

void jet_graph :: operator = (float number)
{

   for (int i = 0; i < x_size; i++)
     for (int j = 0; j < y_size; j++)
	for (int k = 0; k < jet_size; k++)
		vec_array[i][j].elem(k) = number;

}

