# NSL2.8: house.tcl
# AW - 4/95

# nslset neural_network HOUSE	;# model

nslset HOUSE delta 0.05		;# simulation delta
nslset HOUSE end_time 1.0	;# simulation end_time

nslcreate neuron_matrix M 11 11 	;# accommodation layer
nslcreate neuron_matrix S 11 11 	;# disparity layer

nslcreate neuron_vector U 11 	;# inhibition to accommodation layer
nslcreate neuron_vector V 11 	;# inhibition to disparity layer

nslcreate num_vector Ws 3
nslcreate num_vector Wm 3

nslcreate num_data Kmu
nslcreate num_data Ksv
nslcreate num_data Kds
nslcreate num_data Kvs
nslcreate num_data Kms
nslcreate num_data Kam
nslcreate num_data Kum
nslcreate num_data Ksm

nslconnect M A Kam 1	;# M <- + Kam*A
nslconnect M U Kum -1	;# M <- - Kum*U
nslconnect M S Ksm 1	;# M <- + Ksm*S
nslconnect M M Wm 1	;# M <- + Wm*M
nslconnect U M Kmu 1	;# U <- + Kmu*M

nslconnect S D Kds 1	;# S <- + Kds*D
nslconnect S V Kvs -1	;# S <- - Kvs*V
nslconnect S M Kms 1	;# S <- + Kms*M
nslconnect S S Ws 1	;# S <- + Ws*S
nslconnect V S Ksv 1	;# V <- + Ksv*S

nslset M diff_model leaky
nslset M diff_method euler
nslset M diff_delta 0.05 	
nslset M thresh sigma 0.1 1.1 0 1
nslset M tm 0.3			;# nslset data_value tm 1.0

nslset U diff_model leaky
nslset U diff_method euler
nslset U diff_delta 0.05 	
nslset U thresh ramp 0 0 0	
nslset U tm 0.1			;# nslset data_value tm 1.0

nslset S diff_model leaky
nslset S diff_method euler
nslset S diff_delta 0.05 	
nslset S thresh sigma 0.1 1.1 0 1
nslset S tm 0.3			;# nslset data_value tm 1.0

nslset V diff_model leaky
nslset V diff_method euler
nslset V diff_delta 0.05 	
nslset V thresh ramp 0 0 0
nslset V tm 0.1			;# nslset data_value tm 1.0

nslset w value 3.0	 ;# 1/2 of interpupilary distance in cm 
nslset yf value -10.0	 ;# intersection of optical axes  (0,yf)  in cm   	
nslset l value 22.0	 ;# distance of interpupilary line from origin in cm
nslset dmax value 0.25	 ;# maximum disparity
nslset sigma value 0.25	 ;# spread parameter

nslset Ws value 0.25 0.68 0.25
nslset Wm value 0.25 0.68 0.25

nslset Kmu value 1.0
nslset Ksv value 1.0
nslset Kds value 0.5
nslset Kvs value 0.6
nslset Kms value 0.8
nslset Kam value 0.5
nslset Kum value 0.6
nslset Ksm value 0.8

nslset IN single_value 0

# prey

# nslset IN region_single_value 35 35 41 41 1.0
# nslset IN region_single_value 45 45 55 55 1.0

nslset IN region_single_value 0 0 0 0 1.0
nslset IN region_single_value 10 10 10 10 1.0

# barrier

# nslset IN region_single_value 35 35 30 50 1.0

#	nslset_all_layer_file_sector (44,52,45,65)

nslcreate window_interface

# nslset frame_X0 250
# nslset frame_Y0 100
nslset display_frame width 450
nslset display_frame height 600
# nslset frame_rows 1
nslcreate display_frame .house0

nslset display_window layer IN
nslset display_window graph area_level_graph
nslset display_window wymin 0
nslset display_window wymax 1
nslcreate display_window .house0.in

# nslset display_frame X0 700
# nslset display_frame Y0 100
nslset display_frame width 450
nslset display_frame height 300
# nslset display_frame rows 2
nslcreate display_frame .house1

nslset display_window layer U_out
nslset display_window graph spatial_graph_2
nslset display_window wymin 0
nslset display_window wymax 1
# nslset display_window layer_i0 35
# nslset display_window layer_i1 50
nslcreate display_window .house1.u_out

nslset display_window layer V_out
nslset display_window graph spatial_graph_2
nslset display_window wymin 0
nslset display_window wymax 1
# nslset display_window layer_i0 35
# nslset display_window layer_i1 50
nslcreate display_window .house1.v_out

# nslset display_frame X0 700
# nslset display_frame Y0 400
nslset display_frame width 450
nslset display_frame height 300
# nslset display_frame rows 2
nslcreate display_frame .house2

nslset display_window layer M_out
nslset display_window graph spatial_graph_3
nslset display_window wymin -1
nslset display_window wymax 1
nslset display_window sz 10
nslset display_window hidden NSL_ON
# nslset display_window layer_i0 20
# nslset display_window layer_i1 80
# nslset display_window layer_j0 35
# nslset display_window layer_j1 50
nslcreate display_window .house2.m_out

nslset display_window layer S_out
nslset display_window graph spatial_graph_3
nslset display_window wymin -1
nslset display_window wymax 1
nslset display_window sz 10
nslset display_window hidden NSL_ON
# nslset display_window layer_i0 20
# nslset display_window layer_i1 80
# nslset display_window layer_j0 35
# nslset display_window layer_j1 50
nslcreate display_window .house2.s_out

# added because of crashing at the end of execution (!?)

nslrun 1.0
